@@ -18,7 +18,6 @@ import logging as sys_logging
 import os
 
 from eventlet.green import socket
-from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_log import log as logging
 
@@ -84,7 +83,6 @@ service_opts = [
                default=5,
                help=_('Maximum depth allowed when using nested stacks.')),
     cfg.IntOpt('num_engine_workers',
-               default=processutils.get_worker_count(),
                help=_('Number of heat-engine processes to fork and run.'))]
 
 engine_opts = [
@@ -100,6 +98,16 @@ engine_opts = [
                default='trusts',
                help=_('Select deferred auth method, '
                       'stored password or trusts.')),
+    cfg.StrOpt('reauthentication_auth_method',
+               choices=['', 'trusts'],
+               default='',
+               help=_('Allow reauthentication on token expiry, such that'
+                      ' long-running tasks may complete. Note this defeats'
+                      ' the expiry of any provided user tokens.')),
+    cfg.IntOpt('stale_token_duration',
+               default=30,
+               help=_('Gap, in seconds, to determine whether the given token '
+                      'is about to expire.'),),
     cfg.ListOpt('trusts_delegated_roles',
                 default=[],
                 help=_('Subset of trustor roles to be delegated to heat.'
