@@ -177,14 +177,14 @@ def resource_update(context, resource_id, values, atomic_key,
         return bool(rows_updated)
 
 
-def resource_data_get_all(resource, data=None):
+def resource_data_get_all(context, resource_id, data=None):
     """Looks up resource_data by resource.id.
 
     If data is encrypted, this method will decrypt the results.
     """
     if data is None:
-        data = (model_query(resource.context, models.ResourceData)
-                .filter_by(resource_id=resource.id))
+        data = (model_query(context, models.ResourceData)
+                .filter_by(resource_id=resource_id)).all()
 
     if not data:
         raise exception.NotFound(_('no resource data found'))
@@ -1091,8 +1091,15 @@ def purge_deleted(age, granularity='days'):
     stack_del = stack.delete().where(stack.c.deleted_at < time_line)
     engine.execute(stack_del)
     # delete orphaned raw templates
-    stack_templ_sel = sqlalchemy.select([stack.c.raw_template_id])
-    raw_templ_sel = sqlalchemy.not_(raw_template.c.id.in_(stack_templ_sel))
+    raw_templ_sel = raw_template.c.id.in_(
+        sqlalchemy.select([raw_template.c.id]).select_from(
+            sqlalchemy.join(
+                raw_template,
+                stack,
+                sqlalchemy.or_(
+                    stack.c.prev_raw_template_id == raw_template.c.id,
+                    stack.c.raw_template_id == raw_template.c.id),
+                isouter=True)).where(stack.c.id == None))  # noqa
     raw_templ_del = raw_template.delete().where(raw_templ_sel)
     engine.execute(raw_templ_del)
     # purge any user creds that are no longer referenced
