@@ -13,6 +13,7 @@
 
 import abc
 import sys
+import weakref
 
 from keystoneclient import auth
 from keystoneclient.auth.identity import v2
@@ -22,7 +23,6 @@ from keystoneclient import session
 from oslo_config import cfg
 import six
 
-from heat.common import context
 from heat.common.i18n import _
 
 
@@ -38,11 +38,21 @@ class ClientPlugin(object):
     service_types = []
 
     def __init__(self, context):
-        self.context = context
-        self.clients = context.clients
+        self._context = weakref.ref(context)
+        self._clients = weakref.ref(context.clients)
         self._client = None
         self._keystone_session_obj = None
 
+    @property
+    def context(self):
+        ctxt = self._context()
+        assert ctxt is not None, "Need a reference to the context"
+        return ctxt
+
+    @property
+    def clients(self):
+        return self._clients()
+
     @property
     def _keystone_session(self):
         # FIXME(jamielennox): This session object is essentially static as the
@@ -59,9 +69,21 @@ class ClientPlugin(object):
 
         return self._keystone_session_obj
 
+    def invalidate(self):
+        """Invalidate/clear any cached client."""
+        self._client = None
+
     def client(self):
         if not self._client:
             self._client = self._create()
+        elif (cfg.CONF.reauthentication_auth_method == 'trusts'
+                and self.context.auth_plugin.auth_ref.will_expire_soon(
+                    cfg.CONF.stale_token_duration)):
+            # If the token is near expiry, force creating a new client,
+            # which will get a new token via another call to auth_token
+            # We also have to invalidate all other cached clients
+            self.clients.invalidate_plugins()
+            self._client = self._create()
         return self._client
 
     @abc.abstractmethod
@@ -118,10 +140,9 @@ class ClientPlugin(object):
             auth_ref = token_obj.get_auth_ref(self._keystone_session)
 
             if catalog_key in auth_ref:
-                cxt = self.context.to_dict()
-                access_info = cxt['auth_token_info'][access_key]
+                access_info = self.context.auth_token_info[access_key]
                 access_info[catalog_key] = auth_ref[catalog_key]
-                self.context = context.RequestContext.from_dict(cxt)
+                self.context.reload_auth_plugin()
                 url = get_endpoint()
 
         # NOTE(jamielennox): raising exception maintains compatibility with
