@@ -118,6 +118,12 @@ class ResourceInfo(object):
     def matches(self, resource_type):
         return False
 
+    def get_class(self):
+        raise NotImplemented
+
+    def get_class_to_instantiate(self):
+        return self.get_class()
+
     def __str__(self):
         return '[%s](User:%s) %s -> %s' % (self.description,
                                            self.user_resource,
@@ -146,10 +152,23 @@ class TemplateResourceInfo(ResourceInfo):
 
     def get_class(self, files=None):
         from heat.engine.resources import template_resource
+        if files and self.template_name in files:
+            data = files[self.template_name]
+        else:
+            if self.user_resource:
+                allowed_schemes = template_resource.REMOTE_SCHEMES
+            else:
+                allowed_schemes = template_resource.LOCAL_SCHEMES
+            data = template_resource.TemplateResource.get_template_file(
+                self.template_name,
+                allowed_schemes)
         env = self.registry.environment
-        return template_resource.generate_class(str(self.name),
-                                                self.template_name,
-                                                env, files=files)
+        return template_resource.generate_class_from_template(str(self.name),
+                                                              data, env)
+
+    def get_class_to_instantiate(self):
+        from heat.engine.resources import template_resource
+        return template_resource.TemplateResource
 
 
 class MapResourceInfo(ResourceInfo):
@@ -418,23 +437,31 @@ class ResourceRegistry(object):
                 return match
 
     def get_class(self, resource_type, resource_name=None, files=None):
-        if resource_type == "":
-            msg = _('Resource "%s" has no type') % resource_name
-            raise exception.InvalidResourceType(message=msg)
-        elif resource_type is None:
-            msg = _('Non-empty resource type is required '
-                    'for resource "%s"') % resource_name
-            raise exception.InvalidResourceType(message=msg)
-        elif not isinstance(resource_type, six.string_types):
-            msg = _('Resource "%s" type is not a string') % resource_name
-            raise exception.InvalidResourceType(message=msg)
-
         info = self.get_resource_info(resource_type,
                                       resource_name=resource_name)
         if info is None:
             raise exception.ResourceTypeNotFound(type_name=resource_type)
         return info.get_class(files=files)
 
+    def get_class_to_instantiate(self, resource_type, resource_name=None):
+        if resource_type == "":
+            msg = _('Resource "%s" has no type') % resource_name
+            raise exception.StackValidationFailed(message=msg)
+        elif resource_type is None:
+            msg = _('Non-empty resource type is required '
+                    'for resource "%s"') % resource_name
+            raise exception.StackValidationFailed(message=msg)
+        elif not isinstance(resource_type, six.string_types):
+            msg = _('Resource "%s" type is not a string') % resource_name
+            raise exception.StackValidationFailed(message=msg)
+
+        info = self.get_resource_info(resource_type,
+                                      resource_name=resource_name)
+        if info is None:
+            msg = _("Unknown resource Type : %s") % resource_type
+            raise exception.StackValidationFailed(message=msg)
+        return info.get_class_to_instantiate()
+
     def as_dict(self):
         """Return user resources in a dict format."""
         def _as_dict(level):
@@ -581,6 +608,10 @@ class Environment(object):
         return self.registry.get_class(resource_type, resource_name,
                                        files=files)
 
+    def get_class_to_instantiate(self, resource_type, resource_name=None):
+        return self.registry.get_class_to_instantiate(resource_type,
+                                                      resource_name)
+
     def get_types(self,
                   cnxt=None,
                   support_status=None,
