@@ -128,15 +128,10 @@ class Resource(object):
             # Call is already for a subclass, so pass it through
             ResourceClass = cls
         else:
-            from heat.engine.resources import template_resource
-
             registry = stack.env.registry
-            try:
-                ResourceClass = registry.get_class(definition.resource_type,
-                                                   resource_name=name,
-                                                   files=stack.t.files)
-            except exception.TemplateNotFound:
-                ResourceClass = template_resource.TemplateResource
+            ResourceClass = registry.get_class_to_instantiate(
+                definition.resource_type,
+                resource_name=name)
 
             assert issubclass(ResourceClass, Resource)
 
@@ -311,7 +306,7 @@ class Resource(object):
     def reparse(self):
         self.properties = self.t.properties(self.properties_schema,
                                             self.context)
-        self.translate_properties()
+        self.translate_properties(self.properties)
 
     def __eq__(self, other):
         """Allow == comparison of two resources."""
@@ -388,6 +383,19 @@ class Resource(object):
         # common resources have not nested, StackResource overrides it
         return False
 
+    def resource_class(self):
+        """Return the resource class.
+
+        This is used to compare old and new resources when updating, to ensure
+        that in-place updates are possible. This method shold return the
+        highest common class in the hierarchy whose subclasses are all capable
+        of converting to each other's types via handle_update().
+
+        This mechanism may disappear again in future, so third-party resource
+        types should not rely on it.
+        """
+        return type(self)
+
     def has_hook(self, hook):
         # Clear the cache to make sure the data is up to date:
         self._data = None
@@ -412,7 +420,7 @@ class Resource(object):
             return True
         ri = self.stack.env.get_resource_info(self.type(),
                                               self.name)
-        return ri.name == resource_type
+        return ri is not None and ri.name == resource_type
 
     def implementation_signature(self):
         """Return a tuple defining the implementation.
@@ -820,13 +828,12 @@ class Resource(object):
         # save the resource metadata
         self.metadata_set(metadata)
 
-    def translation_rules(self):
+    def translation_rules(self, properties):
         """Return specified rules for resource."""
-        return None
 
-    def translate_properties(self):
+    def translate_properties(self, properties):
         """Translates old properties to new ones."""
-        rules = self.translation_rules() or []
+        rules = self.translation_rules(properties) or []
         for rule in rules:
             rule.execute_rule()
 
@@ -911,6 +918,7 @@ class Resource(object):
         self.regenerate_info_schema(after)
         after_props = after.properties(self.properties_schema,
                                        self.context)
+        self.translate_properties(after_props)
 
         yield self._break_if_required(
             self.UPDATE, environment.HOOK_PRE_UPDATE)
@@ -919,7 +927,6 @@ class Resource(object):
             if not self._needs_update(after, before, after_props, before_props,
                                       prev_resource):
                 return
-
             if not cfg.CONF.convergence_engine:
                 if (self.action, self.status) in (
                         (self.CREATE, self.IN_PROGRESS),
@@ -946,13 +953,20 @@ class Resource(object):
                 self._update_stored_properties()
         except exception.UpdateReplace as ex:
             # catch all UpdateReplace expections
-            if (self.stack.action == 'ROLLBACK' and
-                    self.stack.status == 'IN_PROGRESS'):
-                # handle case, when it's rollback and we should restore
-                # old resource
-                self.restore_after_rollback()
-            else:
-                self.prepare_for_replace()
+            try:
+                if (self.stack.action == 'ROLLBACK' and
+                        self.stack.status == 'IN_PROGRESS'):
+                    # handle case, when it's rollback and we should restore
+                    # old resource
+                    self.restore_after_rollback()
+                else:
+                    self.prepare_for_replace()
+            except Exception as e:
+                # if any exception happen, we should set the resource to
+                # FAILED, then raise ResourceFailure
+                failure = exception.ResourceFailure(e, self, action)
+                self.state_set(action, self.FAILED, six.text_type(failure))
+                raise failure
             raise ex
 
     def prepare_for_replace(self):
@@ -1378,8 +1392,12 @@ class Resource(object):
 
         if not updated_ok:
             ex = exception.UpdateInProgress(self.name)
-            LOG.exception('atomic:%s engine_id:%s/%s' % (
-                rs.atomic_key, rs.engine_id, engine_id))
+            LOG.error(_LE(
+                'Error acquiring lock for resource id:%(resource_id)s with'
+                'atomic key:%(atomic_key)s,'
+                'engine_id:%(rs_engine_id)s/%(engine_id)s') % {
+                    'resource_id': rs.id, 'atomic_key': rs.atomic_key,
+                    'rs_engine_id': rs.engine_id, 'engine_id': engine_id})
             raise ex
 
     def _release(self, engine_id):
