@@ -98,13 +98,13 @@ class KeystoneRoleAssignmentMixin(object):
     def _add_role_assignments_to_group(self, group_id, role_assignments):
         for role_assignment in self._normalize_to_id(role_assignments):
             if role_assignment.get(self.PROJECT) is not None:
-                self.client().client.roles.grant(
+                self.client().roles.grant(
                     role=role_assignment.get(self.ROLE),
                     project=role_assignment.get(self.PROJECT),
                     group=group_id
                 )
             elif role_assignment.get(self.DOMAIN) is not None:
-                self.client().client.roles.grant(
+                self.client().roles.grant(
                     role=role_assignment.get(self.ROLE),
                     domain=role_assignment.get(self.DOMAIN),
                     group=group_id
@@ -113,13 +113,13 @@ class KeystoneRoleAssignmentMixin(object):
     def _add_role_assignments_to_user(self, user_id, role_assignments):
         for role_assignment in self._normalize_to_id(role_assignments):
             if role_assignment.get(self.PROJECT) is not None:
-                self.client().client.roles.grant(
+                self.client().roles.grant(
                     role=role_assignment.get(self.ROLE),
                     project=role_assignment.get(self.PROJECT),
                     user=user_id
                 )
             elif role_assignment.get(self.DOMAIN) is not None:
-                self.client().client.roles.grant(
+                self.client().roles.grant(
                     role=role_assignment.get(self.ROLE),
                     domain=role_assignment.get(self.DOMAIN),
                     user=user_id
@@ -128,13 +128,13 @@ class KeystoneRoleAssignmentMixin(object):
     def _remove_role_assignments_from_group(self, group_id, role_assignments):
         for role_assignment in self._normalize_to_id(role_assignments):
             if role_assignment.get(self.PROJECT) is not None:
-                self.client().client.roles.revoke(
+                self.client().roles.revoke(
                     role=role_assignment.get(self.ROLE),
                     project=role_assignment.get(self.PROJECT),
                     group=group_id
                 )
             elif role_assignment.get(self.DOMAIN) is not None:
-                self.client().client.roles.revoke(
+                self.client().roles.revoke(
                     role=role_assignment.get(self.ROLE),
                     domain=role_assignment.get(self.DOMAIN),
                     group=group_id
@@ -143,13 +143,13 @@ class KeystoneRoleAssignmentMixin(object):
     def _remove_role_assignments_from_user(self, user_id, role_assignments):
         for role_assignment in self._normalize_to_id(role_assignments):
             if role_assignment.get(self.PROJECT) is not None:
-                self.client().client.roles.revoke(
+                self.client().roles.revoke(
                     role=role_assignment.get(self.ROLE),
                     project=role_assignment.get(self.PROJECT),
                     user=user_id
                 )
             elif role_assignment.get(self.DOMAIN) is not None:
-                self.client().client.roles.revoke(
+                self.client().roles.revoke(
                     role=role_assignment.get(self.ROLE),
                     domain=role_assignment.get(self.DOMAIN),
                     user=user_id
@@ -256,30 +256,32 @@ class KeystoneRoleAssignmentMixin(object):
                     self.properties.get(self.ROLES))
 
     def update_assignment(self, prop_diff, user_id=None, group_id=None):
-        (new_role_assignments,
-         removed_role_assignments) = self._find_differences(
-            prop_diff.get(self.ROLES),
-            self._stored_properties_data.get(self.ROLES))
+        # if there is no change do not update
+        if self.ROLES in prop_diff:
+            (new_role_assignments,
+             removed_role_assignments) = self._find_differences(
+                prop_diff.get(self.ROLES),
+                self._stored_properties_data.get(self.ROLES))
 
-        if len(new_role_assignments) > 0:
-            if user_id is not None:
-                self._add_role_assignments_to_user(
-                    user_id,
-                    new_role_assignments)
-            elif group_id is not None:
-                self._add_role_assignments_to_group(
-                    group_id,
-                    new_role_assignments)
+            if len(new_role_assignments) > 0:
+                if user_id is not None:
+                    self._add_role_assignments_to_user(
+                        user_id,
+                        new_role_assignments)
+                elif group_id is not None:
+                    self._add_role_assignments_to_group(
+                        group_id,
+                        new_role_assignments)
 
-        if len(removed_role_assignments) > 0:
-            if user_id is not None:
-                self._remove_role_assignments_from_user(
-                    user_id,
-                    removed_role_assignments)
-            elif group_id is not None:
-                self._remove_role_assignments_from_group(
-                    group_id,
-                    removed_role_assignments)
+            if len(removed_role_assignments) > 0:
+                if user_id is not None:
+                    self._remove_role_assignments_from_user(
+                        user_id,
+                        removed_role_assignments)
+                elif group_id is not None:
+                    self._remove_role_assignments_from_group(
+                        group_id,
+                        removed_role_assignments)
 
     def delete_assignment(self, user_id=None, group_id=None):
         if self._stored_properties_data.get(self.ROLES) is not None:
