@@ -153,7 +153,8 @@ class Subnet(neutron.NeutronResource):
                         ]
                     ),
                 },
-            )
+            ),
+            update_allowed=True
         ),
         TENANT_ID: properties.Schema(
             properties.Schema.STRING,
@@ -245,9 +246,9 @@ class Subnet(neutron.NeutronResource):
         ),
     }
 
-    def translation_rules(self):
+    def translation_rules(self, props):
         return [
-            properties.TranslationRule(self.properties,
+            properties.TranslationRule(props,
                                        properties.TranslationRule.REPLACE,
                                        [self.NETWORK],
                                        value_path=[self.NETWORK_ID])
@@ -311,6 +312,9 @@ class Subnet(neutron.NeutronResource):
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         props = self.prepare_update_properties(json_snippet)
+        if (self.ALLOCATION_POOLS in prop_diff and
+                self.ALLOCATION_POOLS not in props):
+            props[self.ALLOCATION_POOLS] = []
         self.client().update_subnet(
             self.resource_id, {'subnet': props})
 
