@@ -27,12 +27,11 @@ from heat.engine.resources import stack_resource
 from heat.engine import template
 
 
-def generate_class(name, template_name, env, files=None):
-    data = None
-    if files is not None:
-        data = files.get(template_name)
-    if data is None:
-        data = TemplateResource.get_template_file(template_name, ('file',))
+REMOTE_SCHEMES = ('http', 'https')
+LOCAL_SCHEMES = ('file',)
+
+
+def generate_class_from_template(name, data, env):
     tmpl = template.Template(template_format.parse(data))
     props, attrs = TemplateResource.get_schemas(tmpl, env.param_defaults)
     cls = type(name, (TemplateResource,),
@@ -66,6 +65,11 @@ class TemplateResource(stack_resource.StackResource):
         if self.validation_exception is None:
             self._generate_schema(self.t)
 
+    def resource_class(self):
+        # All TemplateResource subclasses can be converted to each other with
+        # a stack update, so allow them to cross-update in place.
+        return TemplateResource
+
     def _get_resource_info(self, rsrc_defn):
         tri = self.stack.env.get_resource_info(
             rsrc_defn.resource_type,
@@ -80,9 +84,9 @@ class TemplateResource(stack_resource.StackResource):
             self.resource_type = tri.name
             self.resource_path = tri.path
             if tri.user_resource:
-                self.allowed_schemes = ('http', 'https')
+                self.allowed_schemes = REMOTE_SCHEMES
             else:
-                self.allowed_schemes = ('http', 'https', 'file')
+                self.allowed_schemes = REMOTE_SCHEMES + LOCAL_SCHEMES
 
         return tri
 
@@ -94,7 +98,7 @@ class TemplateResource(stack_resource.StackResource):
             args = {'name': template_name, 'exc': six.text_type(r_exc)}
             msg = _('Could not fetch remote template '
                     '"%(name)s": %(exc)s') % args
-            raise exception.TemplateNotFound(message=msg)
+            raise exception.NotFound(msg_fmt=msg)
 
     @staticmethod
     def get_schemas(tmpl, param_defaults):
@@ -107,7 +111,7 @@ class TemplateResource(stack_resource.StackResource):
         self._parsed_nested = None
         try:
             tmpl = template.Template(self.child_template())
-        except (exception.TemplateNotFound, ValueError) as download_error:
+        except (exception.NotFound, ValueError) as download_error:
             self.validation_exception = download_error
             tmpl = template.Template(
                 {"HeatTemplateFormatVersion": "2012-12-12"})
@@ -194,7 +198,7 @@ class TemplateResource(stack_resource.StackResource):
             try:
                 t_data = self.get_template_file(self.template_name,
                                                 self.allowed_schemes)
-            except exception.TemplateNotFound as err:
+            except exception.NotFound as err:
                 if self.action == self.UPDATE:
                     raise
                 reported_excp = err
