@@ -120,9 +120,20 @@ class ThreadGroupManager(object):
         """Run the given method in a sub-thread."""
         if stack_id not in self.groups:
             self.groups[stack_id] = threadgroup.ThreadGroup()
-        return self.groups[stack_id].add_thread(self._start_with_trace,
-                                                self._serialize_profile_info(),
-                                                func, *args, **kwargs)
+
+        def log_exceptions(gt):
+            try:
+                gt.wait()
+            except Exception:
+                LOG.exception(_LE('Unhandled error in asynchronous task'))
+            except BaseException:
+                pass
+
+        th = self.groups[stack_id].add_thread(self._start_with_trace,
+                                              self._serialize_profile_info(),
+                                              func, *args, **kwargs)
+        th.link(log_exceptions)
+        return th
 
     def start_with_lock(self, cnxt, stack, engine_id, func, *args, **kwargs):
         """Run the method in sub-thread if acquire a stack lock is successful.
@@ -166,7 +177,7 @@ class ThreadGroupManager(object):
             releasing the lock to avoid race condtitions.
             """
             if stack is not None and stack.action not in (
-                    stack.DELETE, stack.ROLLBACK):
+                    stack.DELETE, stack.ROLLBACK, stack.UPDATE):
                 stack.persist_state_and_release_lock(lock.engine_id)
             else:
                 lock.release()
@@ -715,7 +726,10 @@ class EngineService(service.Service):
             parent_resource_name)
 
         self.resource_enforcer.enforce_stack(stack)
-        stack.store()
+        stack_id = stack.store()
+        if cfg.CONF.reauthentication_auth_method == 'trusts':
+            stack = parser.Stack.load(
+                cnxt, stack_id=stack_id, use_stored_context=True)
         _create_stack_user(stack)
         if convergence:
             action = stack.CREATE
@@ -833,8 +847,11 @@ class EngineService(service.Service):
         # Get the database representation of the existing stack
         db_stack = self._get_stack(cnxt, stack_identity)
         LOG.info(_LI('Updating stack %s'), db_stack.name)
-
-        current_stack = parser.Stack.load(cnxt, stack=db_stack)
+        if cfg.CONF.reauthentication_auth_method == 'trusts':
+            current_stack = parser.Stack.load(
+                cnxt, stack=db_stack, use_stored_context=True)
+        else:
+            current_stack = parser.Stack.load(cnxt, stack=db_stack)
         self.resource_enforcer.enforce_stack(current_stack)
 
         if current_stack.action == current_stack.SUSPEND:
@@ -1195,10 +1212,11 @@ class EngineService(service.Service):
         self.resource_enforcer.enforce(cnxt, type_name)
         try:
             resource_class = resources.global_env().get_class(type_name)
-        except (exception.InvalidResourceType,
-                exception.ResourceTypeNotFound,
-                exception.TemplateNotFound) as ex:
-            raise ex
+        except exception.NotFound:
+            LOG.exception(_LE('Error loading resource type %s '
+                              'from global environment.'),
+                          type_name)
+            raise exception.InvalidGlobalResource(type_name=type_name)
 
         if resource_class.support_status.status == support.HIDDEN:
             raise exception.NotSupported(type_name)
@@ -1240,14 +1258,16 @@ class EngineService(service.Service):
         self.resource_enforcer.enforce(cnxt, type_name)
         try:
             resource_class = resources.global_env().get_class(type_name)
+        except exception.NotFound:
+            LOG.exception(_LE('Error loading resource type %s '
+                              'from global environment.'),
+                          type_name)
+            raise exception.InvalidGlobalResource(type_name=type_name)
+        else:
             if resource_class.support_status.status == support.HIDDEN:
                 raise exception.NotSupported(type_name)
             return resource_class.resource_to_template(type_name,
                                                        template_type)
-        except (exception.InvalidResourceType,
-                exception.ResourceTypeNotFound,
-                exception.TemplateNotFound) as ex:
-            raise ex
 
     @context.request_context
     def list_events(self, cnxt, stack_identity, filters=None, limit=None,
@@ -1803,30 +1823,48 @@ class EngineService(service.Service):
 
     def reset_stack_status(self):
         cnxt = context.get_admin_context()
-        filters = {'status': parser.Stack.IN_PROGRESS}
+        filters = {
+            'status': parser.Stack.IN_PROGRESS,
+            'convergence': False
+        }
         stacks = stack_object.Stack.get_all(cnxt,
                                             filters=filters,
                                             tenant_safe=False,
                                             show_nested=True) or []
         for s in stacks:
-            lock = stack_lock.StackLock(cnxt, s.id, self.engine_id)
-            # If stacklock is released, means stack status may changed.
+            stack_id = s.id
+            lock = stack_lock.StackLock(cnxt, stack_id, self.engine_id)
             engine_id = lock.get_engine_id()
-            if not engine_id:
-                continue
-            # Try to steal the lock and set status to failed.
             try:
-                lock.acquire(retry=False)
+                with lock.thread_lock(retry=False):
+
+                    # refetch stack and confirm it is still IN_PROGRESS
+                    s = stack_object.Stack.get_by_id(
+                        cnxt,
+                        stack_id,
+                        tenant_safe=False,
+                        eager_load=True)
+                    if s.status != parser.Stack.IN_PROGRESS:
+                        lock.release()
+                        continue
+
+                    stk = parser.Stack.load(cnxt, stack=s,
+                                            use_stored_context=True)
+                    LOG.info(_LI('Engine %(engine)s went down when stack '
+                                 '%(stack_id)s was in action %(action)s'),
+                             {'engine': engine_id, 'action': stk.action,
+                              'stack_id': stk.id})
+
+                    # Set stack and resources status to FAILED in sub thread
+                    self.thread_group_mgr.start_with_acquired_lock(
+                        stk,
+                        lock,
+                        self.set_stack_and_resource_to_failed,
+                        stk
+                    )
             except exception.ActionInProgress:
                 continue
-            stk = parser.Stack.load(cnxt, stack=s,
-                                    use_stored_context=True)
-            LOG.info(_LI('Engine %(engine)s went down when stack %(stack_id)s'
-                         ' was in action %(action)s'),
-                     {'engine': engine_id, 'action': stk.action,
-                      'stack_id': stk.id})
-
-            # Set stack and resources status to FAILED in sub thread
-            self.thread_group_mgr.start_with_acquired_lock(
-                stk, lock, self.set_stack_and_resource_to_failed, stk
-            )
+            except Exception:
+                LOG.exception(_LE('Error while resetting stack: %s')
+                              % stack_id)
+                continue
