@@ -20,6 +20,7 @@ import re
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
+from oslo_utils import excutils
 from oslo_utils import uuidutils
 from osprofiler import profiler
 import six
@@ -70,6 +71,27 @@ class ForcedCancel(BaseException):
         return "Operation cancelled"
 
 
+def reset_state_on_error(func):
+    @six.wraps(func)
+    def handle_exceptions(stack, *args, **kwargs):
+        errmsg = None
+        try:
+            return func(stack, *args, **kwargs)
+        except BaseException as exc:
+            with excutils.save_and_reraise_exception():
+                errmsg = six.text_type(exc)
+                LOG.error(_LE('Unexpected exception in %(func)s: %(msg)s'),
+                          {'func': func.__name__, 'msg': errmsg})
+        finally:
+            if stack.status == stack.IN_PROGRESS:
+                msg = _("Unexpected returning while IN_PROGRESS.")
+                stack.state_set(stack.action, stack.FAILED,
+                                errmsg if errmsg is not None else msg)
+                assert errmsg is not None, "Returned while IN_PROGRESS."
+
+    return handle_exceptions
+
+
 @six.python_2_unicode_compatible
 class Stack(collections.Mapping):
 
@@ -718,10 +740,10 @@ class Stack(collections.Mapping):
         self.status_reason = reason
 
         # Persist state to db only if status == IN_PROGRESS
-        # or action == self.DELETE/self.ROLLBACK. Else, it would
+        # or action == UPDATE/DELETE/ROLLBACK. Else, it would
         # be done before releasing the stack lock.
         if status == self.IN_PROGRESS or action in (
-                self.DELETE, self.ROLLBACK):
+                self.UPDATE, self.DELETE, self.ROLLBACK):
             self._persist_state()
 
     def _persist_state(self):
@@ -782,6 +804,7 @@ class Stack(collections.Mapping):
                 r._store()
 
     @profiler.trace('Stack.create', hide_args=False)
+    @reset_state_on_error
     def create(self):
         """Create the stack and all of the resources."""
         def rollback():
@@ -883,6 +906,7 @@ class Stack(collections.Mapping):
                                            (self.status == self.FAILED))
 
     @profiler.trace('Stack.check', hide_args=False)
+    @reset_state_on_error
     def check(self):
         self.updated_time = datetime.datetime.utcnow()
         checker = scheduler.TaskRunner(
@@ -936,6 +960,7 @@ class Stack(collections.Mapping):
             return None
 
     @profiler.trace('Stack.adopt', hide_args=False)
+    @reset_state_on_error
     def adopt(self):
         """Adopt the stack (create stack with all the existing resources)."""
         def rollback():
@@ -955,6 +980,7 @@ class Stack(collections.Mapping):
         creator(timeout=self.timeout_secs())
 
     @profiler.trace('Stack.update', hide_args=False)
+    @reset_state_on_error
     def update(self, newstack, event=None):
         """Update the stack.
 
@@ -1250,6 +1276,8 @@ class Stack(collections.Mapping):
             existing_params.load(newstack.t.env.user_env_as_dict())
             self.t.env = existing_params
             self.t.store(self.context)
+            backup_stack.t.env = existing_params
+            backup_stack.t.store(self.context)
         self.store()
 
         lifecycle_plugin_utils.do_post_ops(self.context, self,
@@ -1399,6 +1427,7 @@ class Stack(collections.Mapping):
         return stack_status, reason
 
     @profiler.trace('Stack.delete', hide_args=False)
+    @reset_state_on_error
     def delete(self, action=DELETE, backup=False, abandon=False):
         """Delete all of the resources, and then the stack itself.
 
@@ -1490,6 +1519,7 @@ class Stack(collections.Mapping):
             self.id = None
 
     @profiler.trace('Stack.suspend', hide_args=False)
+    @reset_state_on_error
     def suspend(self):
         """Suspend the stack.
 
@@ -1515,6 +1545,7 @@ class Stack(collections.Mapping):
         sus_task(timeout=self.timeout_secs())
 
     @profiler.trace('Stack.resume', hide_args=False)
+    @reset_state_on_error
     def resume(self):
         """Resume the stack.
 
@@ -1540,6 +1571,7 @@ class Stack(collections.Mapping):
         sus_task(timeout=self.timeout_secs())
 
     @profiler.trace('Stack.snapshot', hide_args=False)
+    @reset_state_on_error
     def snapshot(self, save_snapshot_func):
         """Snapshot the stack, invoking handle_snapshot on all resources."""
         self.updated_time = datetime.datetime.utcnow()
@@ -1561,6 +1593,7 @@ class Stack(collections.Mapping):
                 scheduler.TaskRunner(rsrc.delete_snapshot, data)()
 
     @profiler.trace('Stack.restore', hide_args=False)
+    @reset_state_on_error
     def restore(self, snapshot):
         """Restore the given snapshot.
 
