@@ -66,6 +66,13 @@ resources:
     type: GenericResourceType
 ''')
 
+hot_tpl_generic_resource_20141016 = template_format.parse('''
+heat_template_version: 2014-10-16
+resources:
+  resource1:
+    type: GenericResourceType
+''')
+
 hot_tpl_generic_resource_all_attrs = template_format.parse('''
 heat_template_version: 2015-10-15
 resources:
@@ -87,6 +94,13 @@ resources:
     type: ResourceWithComplexAttributesType
 ''')
 
+hot_tpl_complex_attrs_20141016 = template_format.parse('''
+heat_template_version: 2014-10-16
+resources:
+  resource1:
+    type: ResourceWithComplexAttributesType
+''')
+
 hot_tpl_mapped_props = template_format.parse('''
 heat_template_version: 2013-05-23
 resources:
@@ -478,6 +492,17 @@ class HOTemplateTest(common.HeatTestCase):
         stack = parser.Stack(utils.dummy_context(), 'test_stack', tmpl)
         snippet = {'list_join': ["\n", {'get_attr': ['rg', 'name']}]}
         self.assertEqual('', self.resolve(snippet, tmpl, stack))
+        # test list_join for liberty template
+        hot_tpl['heat_template_version'] = '2015-10-15'
+        tmpl = template.Template(hot_tpl)
+        stack = parser.Stack(utils.dummy_context(), 'test_stack', tmpl)
+        snippet = {'list_join': ["\n", {'get_attr': ['rg', 'name']}]}
+        self.assertEqual('', self.resolve(snippet, tmpl, stack))
+        # test list join again and update to multiple lists
+        snippet = {'list_join': ["\n",
+                                 {'get_attr': ['rg', 'name']},
+                                 {'get_attr': ['rg', 'name']}]}
+        self.assertEqual('', self.resolve(snippet, tmpl, stack))
 
     def test_str_replace(self):
         """Test str_replace function."""
@@ -654,25 +679,41 @@ class HOTemplateTest(common.HeatTestCase):
         tmpl = template.Template(hot_liberty_tpl_empty)
         self.assertEqual(snippet_resolved, self.resolve(snippet, tmpl))
 
+    def test_list_join_empty_list(self):
+        snippet = {'list_join': [',', []]}
+        snippet_resolved = ''
+        k_tmpl = template.Template(hot_kilo_tpl_empty)
+        self.assertEqual(snippet_resolved, self.resolve(snippet, k_tmpl))
+        l_tmpl = template.Template(hot_liberty_tpl_empty)
+        self.assertEqual(snippet_resolved, self.resolve(snippet, l_tmpl))
+
     def test_join_json(self):
         snippet = {'list_join': [',', [{'foo': 'json'}, {'foo2': 'json2'}]]}
         snippet_resolved = '{"foo": "json"},{"foo2": "json2"}'
-        tmpl = template.Template(hot_liberty_tpl_empty)
-        self.assertEqual(snippet_resolved, self.resolve(snippet, tmpl))
+        l_tmpl = template.Template(hot_liberty_tpl_empty)
+        self.assertEqual(snippet_resolved, self.resolve(snippet, l_tmpl))
+        # old versions before liberty don't support to join json
+        k_tmpl = template.Template(hot_kilo_tpl_empty)
+        exc = self.assertRaises(TypeError, self.resolve, snippet, k_tmpl)
+        self.assertEqual("Items to join must be strings not {'foo': 'json'}",
+                         six.text_type(exc))
 
-    def test_join_type_fail(self):
+    def test_join_object_type_fail(self):
         not_serializable = object
         snippet = {'list_join': [',', [not_serializable]]}
-        tmpl = template.Template(hot_liberty_tpl_empty)
-        exc = self.assertRaises(TypeError, self.resolve, snippet, tmpl)
+        l_tmpl = template.Template(hot_liberty_tpl_empty)
+        exc = self.assertRaises(TypeError, self.resolve, snippet, l_tmpl)
         self.assertIn('Items to join must be string, map or list not',
                       six.text_type(exc))
+        k_tmpl = template.Template(hot_kilo_tpl_empty)
+        exc = self.assertRaises(TypeError, self.resolve, snippet, k_tmpl)
+        self.assertIn("Items to join must be strings", six.text_type(exc))
 
     def test_join_json_fail(self):
         not_serializable = object
         snippet = {'list_join': [',', [{'foo': not_serializable}]]}
-        tmpl = template.Template(hot_liberty_tpl_empty)
-        exc = self.assertRaises(TypeError, self.resolve, snippet, tmpl)
+        l_tmpl = template.Template(hot_liberty_tpl_empty)
+        exc = self.assertRaises(TypeError, self.resolve, snippet, l_tmpl)
         self.assertIn('Items to join must be string, map or list',
                       six.text_type(exc))
         self.assertIn("failed json serialization",
@@ -680,22 +721,39 @@ class HOTemplateTest(common.HeatTestCase):
 
     def test_join_invalid(self):
         snippet = {'list_join': 'bad'}
-        tmpl = template.Template(hot_liberty_tpl_empty)
-        exc = self.assertRaises(TypeError, self.resolve, snippet, tmpl)
+        l_tmpl = template.Template(hot_liberty_tpl_empty)
+        exc = self.assertRaises(TypeError, self.resolve, snippet, l_tmpl)
         self.assertIn('Incorrect arguments', six.text_type(exc))
 
+        k_tmpl = template.Template(hot_kilo_tpl_empty)
+        exc1 = self.assertRaises(TypeError, self.resolve, snippet, k_tmpl)
+        self.assertIn('Incorrect arguments', six.text_type(exc1))
+
+    def test_join_int_invalid(self):
+        snippet = {'list_join': 5}
+        l_tmpl = template.Template(hot_liberty_tpl_empty)
+        exc = self.assertRaises(TypeError, self.resolve, snippet, l_tmpl)
+        self.assertIn('Incorrect arguments', six.text_type(exc))
+
+        k_tmpl = template.Template(hot_kilo_tpl_empty)
+        exc1 = self.assertRaises(TypeError, self.resolve, snippet, k_tmpl)
+        self.assertIn('Incorrect arguments', six.text_type(exc1))
+
     def test_join_invalid_value(self):
         snippet = {'list_join': [',']}
-        tmpl = template.Template(hot_liberty_tpl_empty)
-
-        exc = self.assertRaises(ValueError, self.resolve, snippet, tmpl)
+        l_tmpl = template.Template(hot_liberty_tpl_empty)
+        exc = self.assertRaises(ValueError, self.resolve, snippet, l_tmpl)
         self.assertIn('Incorrect arguments', six.text_type(exc))
 
+        k_tmpl = template.Template(hot_kilo_tpl_empty)
+        exc1 = self.assertRaises(ValueError, self.resolve, snippet, k_tmpl)
+        self.assertIn('Incorrect arguments', six.text_type(exc1))
+
     def test_join_invalid_multiple(self):
         snippet = {'list_join': [',', 'bad', ['foo']]}
         tmpl = template.Template(hot_liberty_tpl_empty)
         exc = self.assertRaises(TypeError, self.resolve, snippet, tmpl)
-        self.assertIn('Incorrect arguments', six.text_type(exc))
+        self.assertIn('must operate on a list', six.text_type(exc))
 
     def test_repeat(self):
         """Test repeat function."""
@@ -1294,6 +1352,7 @@ class StackAttributesTest(common.HeatTestCase):
         self.m.ReplayAll()
 
     scenarios = [
+        # for hot template 2013-05-23, get_attr: hot_funcs.GetAttThenSelect
         ('get_flat_attr',
          dict(hot_tpl=hot_tpl_generic_resource,
               snippet={'Value': {'get_attr': ['resource1', 'foo']}},
@@ -1340,6 +1399,59 @@ class StackAttributesTest(common.HeatTestCase):
                   nested_dict['dict']['a']})),
         ('get_attr_none',
          dict(hot_tpl=hot_tpl_complex_attrs,
+              snippet={'Value': {'get_attr': ['resource1',
+                                              'none',
+                                              'who_cares']}},
+              resource_name='resource1',
+              expected={'Value': None})),
+        # for hot template version 2014-10-16 and 2015-04-30,
+        # get_attr: hot_funcs.GetAtt
+        ('get_flat_attr',
+         dict(hot_tpl=hot_tpl_generic_resource_20141016,
+              snippet={'Value': {'get_attr': ['resource1', 'foo']}},
+              resource_name='resource1',
+              expected={'Value': 'resource1'})),
+        ('get_list_attr',
+         dict(hot_tpl=hot_tpl_complex_attrs_20141016,
+              snippet={'Value': {'get_attr': ['resource1', 'list', 0]}},
+              resource_name='resource1',
+              expected={
+                  'Value':
+                  generic_rsrc.ResourceWithComplexAttributes.list[0]})),
+        ('get_flat_dict_attr',
+         dict(hot_tpl=hot_tpl_complex_attrs_20141016,
+              snippet={'Value': {'get_attr': ['resource1',
+                                              'flat_dict',
+                                              'key2']}},
+              resource_name='resource1',
+              expected={
+                  'Value':
+                  generic_rsrc.ResourceWithComplexAttributes.
+                  flat_dict['key2']})),
+        ('get_nested_attr_list',
+         dict(hot_tpl=hot_tpl_complex_attrs_20141016,
+              snippet={'Value': {'get_attr': ['resource1',
+                                              'nested_dict',
+                                              'list',
+                                              0]}},
+              resource_name='resource1',
+              expected={
+                  'Value':
+                  generic_rsrc.ResourceWithComplexAttributes.
+                  nested_dict['list'][0]})),
+        ('get_nested_attr_dict',
+         dict(hot_tpl=hot_tpl_complex_attrs_20141016,
+              snippet={'Value': {'get_attr': ['resource1',
+                                              'nested_dict',
+                                              'dict',
+                                              'a']}},
+              resource_name='resource1',
+              expected={
+                  'Value':
+                  generic_rsrc.ResourceWithComplexAttributes.
+                  nested_dict['dict']['a']})),
+        ('get_attr_none',
+         dict(hot_tpl=hot_tpl_complex_attrs_20141016,
               snippet={'Value': {'get_attr': ['resource1',
                                               'none',
                                               'who_cares']}},
@@ -1363,8 +1475,16 @@ class StackAttributesTest(common.HeatTestCase):
                 (rsrc.CREATE, rsrc.COMPLETE),
                 (rsrc.RESUME, rsrc.IN_PROGRESS),
                 (rsrc.RESUME, rsrc.COMPLETE),
+                (rsrc.SUSPEND, rsrc.IN_PROGRESS),
+                (rsrc.SUSPEND, rsrc.COMPLETE),
                 (rsrc.UPDATE, rsrc.IN_PROGRESS),
-                (rsrc.UPDATE, rsrc.COMPLETE)):
+                (rsrc.UPDATE, rsrc.COMPLETE),
+                (rsrc.SNAPSHOT, rsrc.IN_PROGRESS),
+                (rsrc.SNAPSHOT, rsrc.COMPLETE),
+                (rsrc.CHECK, rsrc.IN_PROGRESS),
+                (rsrc.CHECK, rsrc.COMPLETE),
+                (rsrc.ADOPT, rsrc.IN_PROGRESS),
+                (rsrc.ADOPT, rsrc.COMPLETE)):
             rsrc.state_set(action, status)
 
             resolved = function.resolve(self.stack.t.parse(self.stack,
@@ -2006,8 +2126,16 @@ class TestGetAttAllAttributes(common.HeatTestCase):
                 (rsrc.CREATE, rsrc.COMPLETE),
                 (rsrc.RESUME, rsrc.IN_PROGRESS),
                 (rsrc.RESUME, rsrc.COMPLETE),
+                (rsrc.SUSPEND, rsrc.IN_PROGRESS),
+                (rsrc.SUSPEND, rsrc.COMPLETE),
                 (rsrc.UPDATE, rsrc.IN_PROGRESS),
-                (rsrc.UPDATE, rsrc.COMPLETE)):
+                (rsrc.UPDATE, rsrc.COMPLETE),
+                (rsrc.SNAPSHOT, rsrc.IN_PROGRESS),
+                (rsrc.SNAPSHOT, rsrc.COMPLETE),
+                (rsrc.CHECK, rsrc.IN_PROGRESS),
+                (rsrc.CHECK, rsrc.COMPLETE),
+                (rsrc.ADOPT, rsrc.IN_PROGRESS),
+                (rsrc.ADOPT, rsrc.COMPLETE)):
             rsrc.state_set(action, status)
 
             if self.raises is not None:
@@ -2017,3 +2145,18 @@ class TestGetAttAllAttributes(common.HeatTestCase):
             else:
                 self.assertEqual(self.expected,
                                  self.resolve(self.snippet, tmpl, stack))
+
+    def test_stack_validate_outputs_get_all_attribute(self):
+        hot_liberty_tpl = template_format.parse('''
+heat_template_version: 2015-10-15
+resources:
+  resource1:
+    type: GenericResourceType
+outputs:
+  all_attr:
+    value: {get_attr: [resource1]}
+''')
+
+        stack = parser.Stack(utils.dummy_context(), 'test_outputs_get_all',
+                             template.Template(hot_liberty_tpl))
+        stack.validate()
