@@ -614,7 +614,7 @@ class ProviderTemplateTest(common.HeatTestCase):
         env_str = {'resource_registry': {'resources': {'fred': {
             "OS::ResourceType": "some_magic.yaml"}}}}
         env = environment.Environment(env_str)
-        ex = self.assertRaises(exception.TemplateNotFound, env.get_class,
+        ex = self.assertRaises(exception.NotFound, env.get_class,
                                'OS::ResourceType', 'fred')
         self.assertIn('Could not fetch remote template "some_magic.yaml"',
                       six.text_type(ex))
@@ -665,7 +665,11 @@ class ProviderTemplateTest(common.HeatTestCase):
 
         env_str = {'resource_registry': {'resources': {'fred': {
             "OS::ResourceType": test_templ_name}}}}
-        env = environment.Environment(env_str)
+        global_env = environment.Environment({}, user_env=False)
+        global_env.load(env_str)
+        with mock.patch('heat.engine.resources._environment',
+                        global_env):
+            env = environment.Environment({})
         cls = env.get_class('OS::ResourceType', 'fred')
         self.assertNotEqual(template_resource.TemplateResource, cls)
         self.assertTrue(issubclass(cls, template_resource.TemplateResource))
@@ -691,10 +695,6 @@ class ProviderTemplateTest(common.HeatTestCase):
             test_templ = test_templ_file.read()
         self.assertTrue(test_templ, "Empty test template")
         self.m.StubOutWithMock(urlfetch, "get")
-        urlfetch.get(test_templ_name,
-                     allowed_schemes=('file',)
-                     ).AndRaise(urlfetch.URLFetchError(
-                         _('Failed to retrieve template')))
         urlfetch.get(test_templ_name,
                      allowed_schemes=('http', 'https')).AndReturn(test_templ)
         parsed_test_templ = template_format.parse(test_templ)
@@ -934,16 +934,18 @@ class TemplateDataTest(common.HeatTestCase):
         self.res.action = self.res.UPDATE
         self.res.nested = mock.MagicMock()
         self.res.get_template_file = mock.Mock(
-            side_effect=exception.TemplateNotFound(
-                message='test_resource.template'))
-        self.assertRaises(exception.TemplateNotFound, self.res.template_data)
+            side_effect=exception.NotFound(
+                msg_fmt='Could not fetch remote template '
+                        '"test_resource.template": file not found'))
+        self.assertRaises(exception.NotFound, self.res.template_data)
 
     def test_template_data_in_create_without_template_file(self):
         self.res.action = self.res.CREATE
         self.res.nested = mock.MagicMock()
         self.res.get_template_file = mock.Mock(
-            side_effect=exception.TemplateNotFound(
-                message='test_resource.template'))
+            side_effect=exception.NotFound(
+                msg_fmt='Could not fetch remote template '
+                        '"test_resource.template": file not found'))
         self.assertEqual('{}', self.res.template_data())
 
 
